@@ -117,11 +117,8 @@ sub _compile {
       # ":nth-*" (with An+B notation)
       $args = _equation($args) if $name =~ /^nth-/;
 
-      # ":first-*" (rewrite to ":nth-*")
-      ($name, $args) = ("nth-$1", [0, 1]) if $name =~ /^first-(.+)$/;
-
-      # ":last-*" (rewrite to ":nth-*")
-      ($name, $args) = ("nth-$name", [-1, 1]) if $name =~ /^last-/;
+      # ":first-*", ":last-*" (rewrite to ":nth-(last-)*")
+      ($name, $args) = ("nth-$+", [0, 1]) if $name =~ /^(?:first-(.+)|(last-.+))$/;
 
       push @$last, ['pc', $name, $args];
     }
@@ -144,7 +141,7 @@ sub _equation {
   return [0, 0] unless my $equation = shift;
 
   # "even"
-  return [2, 2] if $equation =~ /^\s*even\s*$/i;
+  return [2, 0] if $equation =~ /^\s*even\s*$/i;
 
   # "odd"
   return [2, 1] if $equation =~ /^\s*odd\s*$/i;
@@ -241,13 +238,16 @@ sub _pc {
   if (ref $args) {
     my $type     = $class eq 'nth-of-type' || $class eq 'nth-last-of-type' ? $current->[1] : undef;
     my @siblings = @{_siblings($current, $type)};
-    @siblings = reverse @siblings if $class eq 'nth-last-child' || $class eq 'nth-last-of-type';
-
+    my $index;
     for my $i (0 .. $#siblings) {
-      next if (my $result = $args->[0] * $i + $args->[1]) < 1;
-      return undef unless my $sibling = $siblings[$result - 1];
-      return 1 if $sibling eq $current;
+      $index = $i, last if $siblings[$i] eq $current;
     }
+    $index = $#siblings - $index if $class eq 'nth-last-child' || $class eq 'nth-last-of-type';
+    $index++;
+
+    my $delta = $index - $args->[1];
+    return 1 if $delta == 0;
+    return $args->[0] != 0 && ($delta < 0) == ($args->[0] < 0) && $delta % $args->[0] == 0;
   }
 
   # Everything else
